import java.*;
import java.lang.*;
import java.net.*;
import java.util.*;
import java.io.*;

/*
 * This class opens a TCP connection, and allows reading and writing of byte arrays.
 */

public class Xport
{
    //------------- Variables ---------------------
	private Socket mSocket = null;
	private DataInputStream dis = null;
	private DataOutputStream dos = null;

    //------------- Constructors -------------------
    public Xport()
    {
        // We still need to form a connection if this
        // constructor is used. 
    }
    
    public Xport(String ip, int port)
    {
        // we form a connection;
        connect(ip, port);
    }

    //------------- Public Methods -------------------
    public synchronized void connect(String ip, int port)
    {
        InetAddress ipa = null;
        
        // reset all of our variables
        mSocket = null;
        dis = null;
        dos = null;

        try
        {
            ipa = InetAddress.getByName(ip);
        }
        catch(UnknownHostException e)
        {
            System.out.println("Error: " + e.toString());
            return;
        }

        try 
        { 	// Open the socket
            mSocket = new Socket(ipa.getHostAddress(), port);
        }
        catch (IOException e) 
        {
            System.out.println("Error opening socket");
            return;
        }

        try 
        {	// Create an input stream
            dis = new DataInputStream(new BufferedInputStream(mSocket.getInputStream()));
        }
        catch(Exception ex) 
        {
            System.out.println("Error creating input stream");
        }
        try
        {	// Create an output stream
            dos = new DataOutputStream(new BufferedOutputStream(mSocket.getOutputStream()));
        }
        catch(Exception ex) 
        {
            System.out.println("Error creating output stream");
        }
    }
    
    public synchronized void disconnect()
	{
        // we don't try to disconnect if it was never connected
		if (mSocket == null) 
            return;

		try 
        {
			mSocket.close();
            mSocket = null;
		}
	    catch (IOException e){}
	}

	public synchronized void send(byte[] temp)
	{
		try 
        {
			dos.write(temp, 0, temp.length);
			dos.flush();
		}
		catch(Exception ex) 
        {
		    	System.out.println("Error sending data : " + ex.toString());
		}
	}

	public synchronized void send(byte[] temp, int len)
	{
		try 
        {
			dos.write(temp, 0, len);
			dos.flush();
		}
		catch(Exception ex) 
        {
			System.out.println("Error sending data : " + ex.toString());
		}
	}

    // WARNING: this routine may not properly convert Strings to bytes
	public synchronized void send(String given)
	{
		int length = given.length();
		byte[] retvalue = new byte[length];
		char[] c = new char[length];
		given.getChars(0, length, c, 0);
		for (int i = 0; i < length; i++) 
        {
			retvalue[i] = (byte)c[i];
 		}
		send(retvalue);
	}

	public synchronized byte[] receive()
	{
		byte[] retval = new byte[0];

		try 
        {
            if(dis.available() == 0)
                return null;
		}
		catch (IOException e){}
		try 
        {
			retval = new byte[dis.available()];
		} 
        catch (IOException e){}
		try 
        {
			dis.read(retval);
		}
        catch (IOException e){}

		return retval;
	}

	public int available()
	{
		int avail = 0;
		try 
        {
			avail = dis.available();
		} 
        catch (IOException e) {}

		return avail;
	}

    public boolean connected()
    {
        return mSocket != null;
    }
}
